<?php
declare(strict_types=1);

define('ID_INST_ECHOREMOTE', 29103); // id einer aktiven EchoRemote Instanz

//alle Alexas holen
$url  = 'https://{AlexaURL}/api/devices-v2/device?';// . http_build_query($getfields);
$body = json_decode(EchoRemote_CustomCommand(ID_INST_ECHOREMOTE, $url, null, false)['body'], true);

$alexas = [];
foreach ($body['devices'] as $device) {
    $alexas[$device['serialNumber']] = ['accountName' => $device['accountName']];
}

if (count($alexas) === 0) {
    trigger_error('Keine Alexa Geräte gefunden');
}

//alle Appliances holen
$url           = 'https://{AlexaURL}/api/phoenix?';
$body          = EchoRemote_CustomCommand(ID_INST_ECHOREMOTE, $url, null, false)['body'];
$netWorkDetail = (json_decode($body, true))['networkDetail'];
$bridgeDetails =
    json_decode($netWorkDetail, true)['locationDetails']['locationDetails']['Default_Location']['amazonBridgeDetails']['amazonBridgeDetails'];

$appliances = [];
foreach ($bridgeDetails as $skill) {

    foreach ($skill['applianceDetails']['applianceDetails'] as $appliance) {
        $appliances[$appliance['applianceId']] = [
            'applianceType' => $appliance['applianceTypes'][0],
            'friendlyName'  => $appliance['friendlyName'],
            'manufacturer'  => $appliance['manufacturerName']];
    }
}

// alle Gruppen holen und ausgeben
$url             = 'https://{AlexaURL}/api/phoenix/group?';
$body            = EchoRemote_CustomCommand(ID_INST_ECHOREMOTE, $url, null, false)['body'];
$applianceGroups = json_decode($body, true)['applianceGroups'];

foreach ($applianceGroups as $group) {
    echo 'Gruppe: ' . $group['name'] . PHP_EOL . PHP_EOL;

    $groupAppliances = GetGroupAppliances($group['applianceIds'], $alexas, $appliances);

    foreach ($groupAppliances as $appliance) {
        if ($appliance['applianceType'] !== 'Alexa') {
            echo sprintf('%20s: %s (%s)' . PHP_EOL, $appliance['applianceType'], $appliance['friendlyName'], $appliance['manufacturer']);
            $assigned_appliances[$appliance['applianceId']] = true;
        } else {
            echo sprintf('%20s: %s' . PHP_EOL, $appliance['applianceType'], $appliance['friendlyName']);
        }
    }
    echo PHP_EOL . PHP_EOL;
}

//Nicht zugeordnete Appliances ermitteln und ausgeben
foreach ($appliances as $key => $appliance) {
    if (!isset($assigned_appliances[$key])) {
        $notAssignedAppliances[] = $appliance;
    }
}

usort($notAssignedAppliances, 'usort_type');
if (isset($notAssignedAppliances)) {
    echo PHP_EOL . PHP_EOL . 'Nicht zugeordnete Geräte' . PHP_EOL . PHP_EOL;
    foreach ($notAssignedAppliances as $appliance) {
        echo sprintf('%20s: %s (%s)' . PHP_EOL, $appliance['applianceType'], $appliance['friendlyName'], $appliance['manufacturer']);
    }
}

//------------------------------------------------------------------------------------
function GetGroupAppliances(array $applianceIds, array $alexas, array $appliances)
{
    $groupAppliances = [];

    foreach ($applianceIds as $applianceId) {
        $alexa = json_decode($applianceId, true);
        if ($alexa !== null) { //Alexa Gerät
            $groupAppliances[$applianceId] = ['applianceType' => 'Alexa', 'friendlyName' => $alexas[$alexa['dsn']]['accountName']];
        } else {
            $groupAppliances[$applianceId] = [
                'applianceId'   => $applianceId,
                'applianceType' => $appliances[$applianceId]['applianceType'],
                'friendlyName'  => $appliances[$applianceId]['friendlyName'],
                'manufacturer'  => $appliances[$applianceId]['manufacturer']];
        }
    }

    usort($groupAppliances, 'usort_type');
    return $groupAppliances;
}

function usort_type($a, $b)
{
    $typeCmp = strcmp($a['applianceType'], $b['applianceType']);
    if ($typeCmp !== 0) {
        return $typeCmp;
    }

    return strcmp($a['friendlyName'], $b['friendlyName']);
}
